const TJO = require('asksuite-translate-json-object')();
const Translate = require('@google-cloud/translate');

class Translator {
    constructor(INFOCHAT_DATA_SERVER, GOOGLE_TRANSLATE_KEY) {
        this.INFOCHAT_DATA_SERVER = INFOCHAT_DATA_SERVER;
        this.GOOGLE_TRANSLATE_KEY = GOOGLE_TRANSLATE_KEY;

        TJO.init({
            url: this.INFOCHAT_DATA_SERVER + "/",
        });
    }

    static get DEFAULT_LANGUAGE() {
        return 'pt-br';
    }

    static get DEFAULT_ATTRIBUTES() {
        return ['chatTreeId', "dateEnd", "dateInit", "holiday", "trackId", "dialog", "intent", "tags", "goToDialog",
            "type", "search", "image", "registerData", "id", "subject", "to", "default", "hsystemKey", "statusRevision",
            "dialogNewRequest", "compareSuites", "url"];
    }

    tjoInit() {
        return TJO.initMap();
    };

    tjoTranslate(data, language) {

        return new Promise(function (resolve, reject) {

            TJO.translate(data, language, Translator.DEFAULT_ATTRIBUTES)
                .then(function (data) {
                    resolve(data);
                }).catch(function (err) {
                console.log('error ', err);
                reject(data);
            });
        })
    };

    googleTranslateIfNeeded(text, language) {
        const GOOGLE_TRANSLATE_KEY = this.GOOGLE_TRANSLATE_KEY;

        return new Promise(function (resolve, reject) {
            try {
                const definedLanguage = Translator.defineLanguage(language);

                if (definedLanguage === Translator.DEFAULT_LANGUAGE) {
                    resolve({
                        translated: false,
                        translation: text
                    });
                } else {

                    const translate = new Translate({
                        key: GOOGLE_TRANSLATE_KEY,
                    });

                    translate
                        .translate(text, {from: definedLanguage, to: Translator.DEFAULT_LANGUAGE})
                        .then(results => {
                            const translation = results[0];

                            console.log(`Text: ${text}`);
                            console.log(`Translation: ${translation}`);

                            resolve({
                                translated: true,
                                translation
                            });

                        })
                        .catch(err => {
                            console.error('ERROR:', err);
                            reject(err);
                        });
                }
            } catch (e) {
                reject(e);
            }

        });
    };

    static defineLanguage(language) {

        try {
            language = language.toLowerCase();

            if (language.indexOf("en") >= 0) {
                language = "en";
            } else if (language.indexOf("es") >= 0) {
                language = "es";
            } else {
                throw Translator.DEFAULT_LANGUAGE;
            }

        } catch (defaultLang) {
            language = defaultLang;
        }

        return language;
    };
}

module.exports = Translator;
